#!/bin/env perl

use strict;
use warnings;
use feature ':5.10';

use Term::ReadLine;
use Term::ReadLine::Gnu;

my $term = Term::ReadLine::Gnu->new("parse-decklist",\*STDIN,\*STDERR);
$| = 1;

my %csv;
my %canon;

sub decode
{
  my $w = shift;
  my $for_csvid = shift;
  $w =~ tr/\340\341\342\351\355\366\372\373/aaaeiouu/;
  if ($for_csvid // 0)
    {
      # en- and em-dashes become underscores, and AE becomes E
      $w =~ tr/\226\227\306/__E/;
    }
  else
    {
      # en- and em-dashes become hyphens, and AE becomes AE
      $w =~ tr/\226\227/--/;
      $w =~ s/\306/AE/g;

      # parse curly quotes and bullets.
      $w =~ tr/\221\222\223\224\225/''""*/;
    }
  return $w;
}

open(my $manalink_csv, '<', 'Manalink.csv') or die "open Manalink.csv: $!";
while (<$manalink_csv>)
  {
    s/[\n\r]+$//;
    my @fields = split(/;/);
    next if ($fields[0] =~ /^(ID|#)$/);
    die "[$_]" unless $fields[0] =~ /^[0-9]+$/;
    $fields[1] =~ s/""/"/g;
    $canon{$fields[0]} = $fields[1];
    $fields[1] =~ s@ */+ *@ // @g;
    $csv{lc $fields[1]} = $fields[0];
    my $dec = decode($fields[1]);
    if ($dec ne $fields[1])
      {
	$csv{lc $dec} = $fields[0];
      }
  }
close $manalink_csv or die "close Manalink.csv: $!";

foreach my $filename (@ARGV)
  {
    print "$filename: ";
    $filename =~ /\.dck$/ or die "filename doesn't end in .dck";
    $filename =~ /^([0-9]{4})-([0-9]{2})-([0-9]{2})-(.*)\.dck$/ or die "Couldn't parse date/stem";
    my $yyyy = $1;
    my $mm = $2;
    my $dd = $3;
    my $stem = $4;

    open(my $f, '<', $filename) or die "open: $!";
    my $insb = 0;
    my $lastempty = 0;
    my $num_main = 0;
    my $num_sb = 0;
    my $maindeck = '';
    if ($stem =~ /^EDH/)
      {
	my $nam = '* Elder Dragon Highlander';
	$maindeck = '.' . $csv{lc $nam} . "\t1\t" . $nam . "\r\n";
      }

    while (my $ln = <$f>)
      {
	$ln =~ s/[ \t\n\r]+$//;
	$ln =~ s/^[ \t]+//;
	if ($ln eq '')
	  {
	    die "already in sideboard" if $insb;
	    if (!$lastempty)
	      {
		$lastempty = 1;
	      }
	    else
	      {
		$insb = 1;
		$lastempty = 0;
	      }
	    next;
	  }
	die "single empty line" if $lastempty;
	$ln =~ /^([0-9]+)\s+(.+)$/ or die "Couldn't parse line [$ln]";
	my $num = $1;
	$num > 0 or die "[$ln]: less than 1 card";
	my $nam = $2;
	$nam =~ s@\s*/+\s*@ // @g;
	my $id = $csv{lc $nam};
	defined $id or die "Unknown csv for [$nam]";
	if ($insb)
	  {
	    $num_sb += $num;
	  }
	else
	  {
	    $num_main += $num;
	    $maindeck .= '.' . $id . "\t" . $num . "\t" . $canon{$id} . "\r\n";
	  }
      }
    close $f or die "close: $!";

    print "maindeck:$num_main\tsideboard:$num_sb\n";
    my $rename = undef;
    while (!(defined $rename) || (length $rename > 28) || -f "output/${rename}.dck" || -f "already/${rename}.dck")
      {
	$rename //= $stem;
	print "1234567890123456789012345678\n";
	$rename = $term->readline('', $rename);
	if (-f "output/${rename}.dck") { say "Already exists in output/"; }
	if (-f "already/${rename}.dck") { say "Already exists in already/"; }
      }
    open(my $o, '>', "output/${rename}.dck") or die qq'open "output/${rename}.dck": $!';
    print $o ";$rename\r\n";
    print $o ";\r\n";
    print $o ";\r\n";
    print $o ";User E-Mail\r\n";
    print $o ";$dd/$mm/$yyyy\r\n";
    print $o ";1\r\n";
    print $o ";4th Edition\r\n";
    print $o ";Daily Deck List $stem\r\n";
    print $o "\r\n";
    print $o $maindeck;
    close $o or die qq'close "output/${rename}.dck": $!';
    rename $filename, "done/$filename" or die qq'rename("$filename", "done/$filename"): $!';
    print "\n";
  }
